local MAJOR, MINOR = "TradeLinkDecoder-1.0", 1
local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

local bitband, tostring, strlen, strbyte, wipe, type = bit.band, tostring, string.len, string.byte, table.wipe, type

lib.spellData = {}

local encodedByte = {
	'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
	'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z',
	'0','1','2','3','4','5','6','7','8','9','+','/'
	}

local decodedByte = {}

for i=1,#encodedByte do
	local b = strbyte(encodedByte[i])
	decodedByte[b] = i - 1
end

function lib:Decode(link, noBoP, useBlackList, t)
	local _, _, _, _, bitmap, data = self:DisasembleLink(link)
	if not data or not bitmap then
		geterrorhandler( MAJOR.." No data or bitmap returned from "..tostring(link) )
		return t
	end
	t = t or {}
	if t and type(t) == "table" then
		t = wipe(t)
	end 
-------Decode Function from Lilsparky's Gnomish Yellow Pages :)
	local index = 1
	for i=1, strlen(bitmap) do
		local b = decodedByte[strbyte(bitmap, i)]
		local v = 1

		for j=1,6 do
			if index <= #data then
				if bitband(v, b) == v then
					t[ data[index]  ] = true
				end
			end
			v = v * 2
			index = index + 1
		end
	end
--------Other Methods taken from Maldiva's "TradeLinks" addon and "LibTradeLinks-1.0".
	if noBoP then
		self:Sanitize( "noBoP", t)
	end
	if useBlackList then
		self:Sanitize( "BlackList", t)
	end
	return t
end

function lib:DisasembleLink(link)
	local spellID, curLvl, maxLvl, guid, bitmap = link:match("trade:(%d+):(%d+):(%d+):([0-9a-fA-F]+):([A-Za-z0-9+/]+)")
	spellID, currLvl, maxLvl, guid = tonumber(spellID), tonumber(currLvl), tonumber(maxLvl), tonumber(guid)
	local data = self.spellData[spellID]
	if data and type(data) == "table" then
		return spellID, curLvl, maxLvl, guid, bitmap, data
	end
end

function lib:AddData(spellList, data)
	for i = 1, #spellList do
		self.spellData[ spellList[i] ] = data
	end
	if type(self.BlackList) == "table" then
		local bl = self.BlackList
		for i=1, #spellList do
			tinsert(bl, spellList[i])
		end
	end
end

function lib:SetBlackList( list )
	self.BlackList = list
end

function lib:SetBoPList( list )
	self.BoPList = list
end

function lib:Sanitize( class , t)
	if class == "noBoP" then
		local bop = self.BoPList
		for i = 1, #bop do
			t[ bop[i] ] = nil
		end
	elseif class == "BlackList" then
		local bl = self.BlackList
		for i = 1, #bl do
			t[ bl[i] ] = nil
		end
	end
	return t
end

function lib:LotsOfData( link )
	local spellID, curLvl, maxLvl, guid, bitmap, _ = self:DisasembleLink( link )
	local spellList = self:DumpSpells( link )
	local profName = ( GetSpellInfo(spellID) )
	return profName, spellID, curLvl, maxLvl, guid, bitmap, spellList
end

function lib:GetBuildVersion()
	return "9551"
end
