local rev = tonumber(("$Revision: 179 $"):match("(%d+)")) or 0
if rev > GuildCraft.version then GuildCraft.version = rev end

local D
do
	function D(...)
		local str
		local arg = select(1, ...) or ""
		if string.find(arg, "%%") then
			str = (select(1, ...)):format(select(2,...))
		else
			str = string.join(", ", tostringall(...))
			str:gsub(":,", ":")
		end
		if (not GuildCraft ) or (not GuildCraft.db.global.debug.comms) then
			return str
		end

		ChatFrame1:AddMessage("|cff33FF66GC~WotLK-ChatComms:|r: "..str)
	end
end

local AddonParent = LibStub("AceAddon-3.0"):GetAddon("GuildCraft")
local module = AddonParent:NewModule("ChatChannelComms", "AceSerializer-3.0")
local CommsIO = AddonParent:GetModule("CommsIO")
module.rev = rev

local CTL = assert(ChatThrottleLib, "Many things in GuildCraft will require ChatThrottleLib, if it's not there you installed this wrong, very wrong!")

local channelName, channelPass, channelNumber
local function CheckChatChannel()
	if not channelName then D("CCC - No Channel Name") return false end
	D("CCC", GetChannelName(channelName))	
	if GetChannelName(channelName) == 0 then
		local sucsess = JoinChannelByName(channelName, channelPass)
		if not sucsess then
			message("GuildCraft -- Unable to Join ChatChannel, too many chat channels in use. Please Leave one or disable Chat Comms" )
			return false
		end
	end
	channelNumber = GetChannelName(channelName)
	return true
end

--[[
self.db = 
AddonParent.db.profile.chatchannelcomms = {
			channel = "GuildCraftGlobal",
			password = "",
			},
]]--

function module:OnEnable()
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", module.ChatFilter)
	--self:Print("Chat Filter is not applied, so you will see channel spam if it is enabled for this frame")	
	self.db = AddonParent.db.profile.chatchannelcomms
	self.char = AddonParent.db.char
	LibStub("AceEvent-3.0").RegisterMessage(self, "GuildCraft_RosterUpdate")
	
end

function module.ChatFilter(incMsg)
	if not CheckChatChannel() then return end
	if arg9 and arg9:lower() == (channelName or ""):lower() then
		return true
	end
	if incMsg:find("^[\001-\003]") then
		return true
	end
end

function module:GuildCraft_RosterUpdate(event)
	D(event)
	self:InitOptions()
	self:InitChatChannelGreeting()
end

function module:InitOptions()
	if AddonParent.db.char.shareToChat then
		if self.db.channel then
			channelName = self.db.channel
			channelPass = self.db.password
			D("InitOpts", channelName, channelPass, "//", self.db.channel, self.db.password)
			CheckChatChannel()
		end
	end
end

function module:InitChatChannelGreeting()
	self:SendChatMessage("Call_For_Update")
end

function module:Join()
	D("Join?")
end

do
	---Based on AceComms-3.0 & Adapted to use ChatChannels instead.
	local MSG_MULTI_FIRST, MSG_MULTI_NEXT, MSG_MULTI_LAST, prio, prefix = "\001", "\002", "\003", "BULK", "GuildCraft-CCC"
	local type, strfind, strsub = _G.type, _G.strfind, _G.strsub

	function module:SendChatMessage(...)
		if not CheckChatChannel() then return end
		D("Send to Chat: ", ...)
		local text = self:Serialize(...):gsub("|", "||")
		if not type(text)=="string" then
			error('GuildCraft-ChatComms --Usage: SendCommMessage(...) --where ... is a serilizeable value')
		end
		if strfind(text, "[\001-\003]") then
			error("GuildCraft-ChatComms \\001 - \\003 bits are reserved for protocol use, try again")
		end

		local textlen = #text
		local maxtextlen = 254 --255 is the max for chat messages, but 254 works also, multipart section accounts for header bit.

		if textlen <= maxtextlen then
		-- fits all in one message
			CTL:SendChatMessage(prio, prefix, text, "CHANNEL", nil, channelNumber )
		else
			maxtextlen = maxtextlen - 1 -- 1 extra byte for part indicator in prefix
		-- First Part
			local chunk = strsub(text, 1, maxtextlen)
			CTL:SendChatMessage(prio, prefix, MSG_MULTI_FIRST..chunk, "CHANNEL", nil, channelNumber, channelName )
		-- Middle Parts
			local pos = 1+maxtextlen
			while pos+maxtextlen <= textlen do
				chunk = strsub(text, pos, pos+maxtextlen-1)
				CTL:SendChatMessage(prio, prefix, MSG_MULTI_NEXT..chunk, "CHANNEL", nil, channelNumber, channelName )
				pos = pos + maxtextlen
			end
		-- Last
			chunk = strsub(text, pos)
			CTL:SendChatMessage(prio, prefix, MSG_MULTI_LAST..chunk, "CHANNEL", nil, channelNumber, channelName )
		end
	end
end

do
	local cache = setmetatable({},{__mode = "kv"})
	local function newtable()
		local t = next(cache)
		if t then
			cache[t] = nil
		else
			t = {}
		end
		return t
	end
	local function delTable(t)
		if type(t) == "table" then
			wipe(t)
			cache[t] = true
			return nil
		end
		return nil
	end

	local incMsgHold = {}
	module.incMsgHold = incMsgHold
	local function OnCommsEvent(msg, author, lang, fullName, target, status, zoneID, chlNumber, chlName)
		if strfind(msg, "^[\001-\003]") then -- this is multi part
			if strfind(msg, "^\001") then -- this is first part
				incMsgHold[author] = newtable()
				msg = msg:gsub("^\001","")
				tinsert(incMsgHold[author], msg)
			elseif strfind(msg, "^\002") and (incMsgHold[author] and incMsgHold[author][1]) then -- this is 2nd part
				msg = msg:gsub("^\002","")
				tinsert(incMsgHold[author], msg)
			elseif strfind(msg, "^\003") and (incMsgHold[author] and incMsgHold[author][1]) then -- this is Last part
				msg = msg:gsub("^\003","")
				tinsert(incMsgHold[author], msg)
				CommsIO.DispatchComms( author, module:Deserialize( table.concat(incMsgHold[author],""):gsub("||", "|") ) )
				incMsgHold[author] = delTable(t)
			end
		else
			D("OnCommsEvent", module:Deserialize(msg:gsub("||", "|")))
			CommsIO.DispatchComms( author, module:Deserialize(msg:gsub("||", "|")) )
		end
	end

	local frame = CreateFrame("Frame")
	frame:UnregisterAllEvents()
	frame:RegisterEvent("CHAT_MSG_CHANNEL")
	frame:SetScript("OnEvent", function(frame, event, ...)
			if not AddonParent.hasUpdateRoster then return end
			if not channelName then return end
			if (select(9, ...)):lower() == channelName:lower() then
				local author = select(2, ...)
				if AddonParent.online[author] then 
					D(event, author, " // sender in guild, ignoring")
					return
				end	--Guild Traffic is sent over guild addon channel, we do not proccess messages from guild in the chat channel.
				OnCommsEvent(...)
				return 
			end
		end)
	module.frame = frame
end


