--Core.lua
--$Date: 2009-02-10 22:06:37 +0000 (Tue, 10 Feb 2009) $ / $Revision: 207 $
--[[
Copyright (c) 2007 by Orionshock

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

All files included in this archive other than the included librarys, an assortment of Ace3 Core Librarys and LibGratuity,  are licensed  under the above.
All other files are licensed under their respective licenses.

]]--

local rev = tonumber(("$Revision: 207 $"):match("(%d+)")) or 0
local lastAddonDBCompatVersion = 184
--local lastAddonDBCompatVersion = 7500	--WowaceDistro Option

--Debug and table functions...
local D, newTable, delTable, clearTable
do
	function D(...)
		local str
		local arg = select(1, ...) or ""
		if string.find(arg, "%%") then
			str = (select(1, ...)):format(select(2,...))
		else
			str = string.join(", ", tostringall(...))
			str:gsub(":,", ":")
		end
		if (not GuildCraft ) or (not GuildCraft.db.global.debug.core) then
			return str
		end

		ChatFrame1:AddMessage("|cff33FF66GC~WotLK-Core:|r: "..str)
	end
end

do
-- Table Pool for recycling tables
	local tablePool = {}
	setmetatable(tablePool, {__mode = "kv"});	-- Weak table

	function newTable()
		local t = next(tablePool) or {};
		tablePool[t] = nil;
		return t;
	end

	function delTable(t)
		if (type(t) == "table") then
			for k, v in pairs(t) do
				if (type(v) == "table") then
					delTable(v);	-- child tables get put into the pool
				end
				t[k] = nil;
			end
			setmetatable(t, nil);
			t[true] = true;
			t[true] = nil;
			tablePool[t] = true;
		end
		return nil;
	end

	function clearTable(t, keepRoot)
		for k,v in pairs(t) do
			if keepRoot and type(v) == 'table' then
				t[k] = delTable(t[k])
			else
				t[k] = nil
			end
		end
		return t
	end
end


---DB Structure..
local masterDB_Defaults = {
	char = {
		shareToGuild = true,
		shareToChat = false,
		links = {},
	},	--End Char DBO
	profile = {
		moduleStatus = {
			OutOfGuild = false,
			},
		displaySortMethod = "Name",
		ShowExtTT = false,
		chatSpam = false,
		quickListOutOfGuild = {
			},
		chatchannelcomms = {
			channel = "GuildCraftGlobal",
			password = "",
			},
		},
	factionrealm = {
		links = {},

	}, -- End of faction table
	global = {
		debug = {
			core = false,
			tradeskill = false,
			dbo = false,
			comms = false,
			browser = false,
			outofguild = false,
			fubar = false,
		},
		playerVersions = {
		},
	},
}

--[[
Core:
	Handles parent Option Table
	Handles AceDB Object
 ]]--

GuildCraft = LibStub("AceAddon-3.0"):NewAddon("GuildCraft", "AceConsole-3.0", "AceEvent-3.0")
GuildCraft.version = (tonumber(("$Revision: 207 $"):match("(%d+)")) or 0 )
local core = GuildCraft
local L = LibStub("AceLocale-3.0"):GetLocale("GuildCraft")

local online, offline, officers = newTable(), newTable(), newTable()
core.online, core.offline = online, offline

--local L = setmetatable({}, {__index = function(t,k,v)
--		t[k] = k
--		return k
--	end})
--core.L = L



function core:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("GuildCraft3" , masterDB_Defaults )
	if self.db.global.AddonDBVersion and (tonumber(self.db.global.AddonDBVersion) < lastAddonDBCompatVersion) then
		self.db:ResetDB()
		message(L["Database reset, You may need to reload your UI"])
	end
	if (not self.db.global.AddonDBVersion) then	
		self:Print(L["Loaded, /guildcraft to bring up config, /gcb for the browser. Version: %s"], GuildCraft.version)
	end
	self.db.global.AddonDBVersion = self.version
	self.Cache = {
			GuildTrades = {},
			main = {},
		}
--	self.db.global.L = L
end

function core.initBrowser(input)
	D("gcb Called")
	if IsAddOnLoaded("GuildCraft_Browser") or core:GetModule("BrowserUI", true) then
		D("browser loaded and found, opening")
		core:GetModule("BrowserUI"):ToggleBrowser()
	else
		D("Browser not enabled / loaded, enabling")
		EnableAddOn("GuildCraft_Browser")
		D("Enabling Browser")
		local loaded, reason = LoadAddOn("GuildCraft_Browser")
		if not loaded then
			core:Print(L["Cannot load browser UI reason: %s"], reason)
		else
			if core:GetModule("BrowserUI", true) then
				D("Found and Loading")
				local l = core:GetModule("BrowserUI").CreateBrowser and core:GetModule("BrowserUI"):ToggleBrowser()
			elseif not core:GetModule("BrowserUI", true) then
				message("Browser Not loaded, see previous errors")
			end
		end
	end
end

function core:OnEnable()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Guild Craft", self.optionsTable)
	self:RegisterChatCommand("guildcraft", function()
		LibStub("AceConfigDialog-3.0"):Open("Guild Craft")
	end )
	
	self:RegisterChatCommand("gcb", self.initBrowser )
	GuildRoster()
	--self:RegisterEvent("GUILD_ROSTER_UPDATE")
	self.version = self.version.."-Curse-Distro"

	D("Guild Craft Core Loaded")
end


local prefix = "|cff33FF66"..L["Guild Craft"]..":|r "
local proto = {}
function proto.Print(self, match, ...)
	if (core.db) and (core.db.profile.chatSpam) then
		if string.find(match, "%%") then
			DEFAULT_CHAT_FRAME:AddMessage(prefix..string.join(", ", match:format(...) ) )
		else
			DEFAULT_CHAT_FRAME:AddMessage(prefix..string.join(", ", match, ... ) )
		end
	end
end

core.Print = proto.Print
core:SetDefaultModulePrototype(proto)


do
	local delay, interval = 0, 30
	local function onUpdateFunc(frame, elapsed)
		delay = delay + elapsed
		if delay > interval then
			if IsInGuild() then
	--			D("GRU", count)
				local offlineStatus = GetGuildRosterShowOffline()
	--			D("GRU~GetGuildRosterShowOffline()", count)
				SetGuildRosterShowOffline(true)
	--			D("SetGuildRosterShowOffline(true)", count)
				for i = 1, GetNumGuildMembers(true) do
					local name, _, _, _, _, _, _, _, oline = GetGuildRosterInfo(i);
					if name then
						online[name] = oline and true or nil
						offline[name] = not oline and true or nil
					end
				end
				SetGuildRosterShowOffline(offlineStatus)
	--			D("SetGuildRosterShowOffline(status)", count)
			end
			if not core.hasUpdateRoster then
				core.hasUpdateRoster = true
				core:SendMessage("GuildCraft_RosterUpdate")
			end
			delay = 0
		end
			
	end
	local frame = CreateFrame("Frame")
	frame:SetScript("OnUpdate", onUpdateFunc)
	core.frame = frame
	frame:Show()
end
--
