local rev = tonumber(("$Revision: 180 $"):match("(%d+)")) or 0
if rev > GuildCraft.version then GuildCraft.version = rev end

local D, newTable, delTable, clearTable

do
	function D(...)
		local str
		local arg = select(1, ...) or ""
		if string.find(arg, "%%") then
			str = (select(1, ...)):format(select(2,...))
		else
			str = string.join(", ", tostringall(...))
			str:gsub(":,", ":")
		end
		if (not GuildCraft ) or (not GuildCraft.db.global.debug.core) then
			return str
		end

		ChatFrame1:AddMessage("|cff33FF66GC~WotLK-Opts:|r: "..str)
	end
end

do
-- Table Pool for recycling tables
	local tablePool = {}
	setmetatable(tablePool, {__mode = "kv"});	-- Weak table

	function newTable()
		local t = next(tablePool) or {};
		tablePool[t] = nil;
		return t;
	end

	function delTable(t)
		if (type(t) == "table") then
			for k, v in pairs(t) do
				if (type(v) == "table") then
					delTable(v);	-- child tables get put into the pool
				end
				t[k] = nil;
			end
			setmetatable(t, nil);
			t[true] = true;
			t[true] = nil;
			tablePool[t] = true;
		end
		return nil;
	end
	
	function clearTable(t)
		for k,v in pairs(t) do
			if type(v) == 'table' then
				t[k] = delTable(t[k])
			else
				t[k] = nil
			end
		end
		return t
	end
end

local AddonParent = LibStub("AceAddon-3.0"):GetAddon("GuildCraft")
local module = AddonParent:NewModule("OptionsDebug")
module.rev = rev
local L = LibStub("AceLocale-3.0"):GetLocale("GuildCraft")
--local L = AddonParent.L
local BTS = LibStub("Mini-Bable-TradeSkill")

local function noop() end

AddonParent.optionsTable = function ()
	local t = {
		type = "group",
		name = L["Guild Craft"],
		handler = AddonParent,
		childGroups = "tab",
		args = {
			main = module:MainDialouge(),
			outOfGuild = AddonParent:GetModule("Non-Guild-Comms", true) and AddonParent:GetModule("Non-Guild-Comms"):GetModuleOptions() or nil,			
			dboFuncs = module:GetDBO_Options(),
			enableDebug = module:GetDebugOptions(),
		},
	}
	return t
end



-------------
--Main Config Window
------------

function module:MainDialouge()
	local t = {
		type = "group",
		name = "Guild Craft Main Control Panel",
		handler = module,
		order = 1,
		args = {
			sendToGuild = { type = 'execute', name = L["Publish To Guild"], order = 1,
				func = function()
					--module:UpdateVersionHash()
					if AddonParent:GetModule("CommsIO",true) then
						AddonParent:GetModule("DBO"):PublishToGuild()
						module:Print(L["Tradeskills Sent to Guild"])
					end
				end,
			},
			resetDB = { type = 'execute', name = L["Reset Entire Database"] , order = 2, confirm = true,
				func = function()
						AddonParent.db:ResetDB()
						D("Database Reset")
						ReloadUI()
					end,
				--hidden = function(info) return not AddonParent.db.global.MasterFlag end, Always show this
				confirmText = L["conferm_reset"],
			},
			shareToGuild = { type = "toggle", name = L["Share Tradeskills with Guild / Others"], order = 3, width = "full",
				get = function(info) return AddonParent.db.char.shareToGuild end,
				set = function(info, value) AddonParent.db.char.shareToGuild = value end,
			},
			stc = { type = "group", name = L["Share With Custom / Defualt Chat channel"], order = 3, inline = true,
				args = {
					shareToChat = { type = "toggle", name = L["Share Tradeskills with Chat Channel"], order = 1, width = "full",
						get = function(info) return AddonParent.db.char.shareToChat end,
						set = function(info, value) AddonParent.db.char.shareToChat = value end,
					},
					channel = { type = "input", name = CHANNEL_CHANNEL_NAME , order = 2,
						get = function(info) return AddonParent.db.profile.chatchannelcomms.channel end,
						set = function(info, value)
								if value == "" then return end
								AddonParent.db.profile.chatchannelcomms.channel = value
							 end,
						disabled = function(info) return not AddonParent.db.char.shareToChat end,
						},
					pass = { type = "input", name = PASSWORD, order = 3,
						get = function(info) return AddonParent.db.profile.chatchannelcomms.password end,
						set = function(info, value)
								if value == "" then return end
								AddonParent.db.profile.chatchannelcomms.password = value
							 end,
						disabled = function(info) return not AddonParent.db.char.shareToChat end,
						},
					push = { type = "execute", name = JOIN, order = 4, func = function() 
							local m = AddonParent:GetModule("ChatChannelComms", true)
							if m then m:InitOptions(); m:InitChatChannelGreeting() end end,
						disabled = function(info) return not AddonParent.db.char.shareToChat end,
						}
					},
				},
			extTT = { type = "toggle", name = L["Show Extended Tooltip"], order = 4, width = "full",
				get = function(info) return AddonParent.db.profile.ShowExtTT end,
				set = function(info, value) AddonParent.db.profile.ShowExtTT = value end,
			},
			chatSpam = {type = "toggle", name = L["Show Chat Spam"], order = 5, width = "full",
				get = function(info) return AddonParent.db.profile.chatSpam end,
				set = function(info, value) AddonParent.db.profile.chatSpam = value end,},
			spacer1 = { type = "description", name = "\n\n", order = 50},
			version = { type = "description", name = L["Version & Release: %s, %s"]:format(AddonParent.version, "v3.3.9 Beta"), order = 100,
				desc = "If you see 'v3.3.9 Beta' here, that means you prolly have a SVN checkout, this is normal"
			}
		},
	}
	local tts = AddonParent:GetModule("TradeSkillScanner", true)
	if tts then
		local index = 50
		for i,v in pairs(tts.links) do
			t.args[tostring(i)] = { name = L["%s is being shared\n"]:format(v), order = index+1 , type = "description" }
			index = index + 1
		end
	end
	return t
end

--------------------
--Debug Options
-------------------

function module:GetDebugOptions()
	local t = { 
		name = "Enable Debug", type = "group", order = -9, childGroups = "tab", handler = module,get = "GetDebug", set = "ToggleDebug",
		args = {
			core = { name = "Core", type = "toggle", order = 1},
			tradeskill = { name = "Tradeskill Module", type = "toggle", order = 2},
			dbo = { name = "Database Module", type = "toggle", order = 3 },
			comms = { name = "Communications Module", type = "toggle", order = 4 },
			browser = { name = "Browser UI Module", type = "toggle", order = 5 },
			outofguild = { name = "Out of Guild Module", type = "toggle", order = 6 },
			fubar = { name = "FuBar Module", type = "toggle", order = 7 },
			all = { name = "Enable All", type = "execute", order = 8, func = "EnableAllDebug" },
			none = { name = "Disable All", type = "execute", order = 9, func = "DisableAllDebug" },
			MF = {	type = "toggle", name = "Master Flag", order = 11,
				get = function() return AddonParent.db.global.MasterFlag end,
				set = function(_, status) AddonParent.db.global.MasterFlag = status end,
				hidden = true,
			},
		}
	}
	return t
end

function module:GetDebug(info)
	return AddonParent.db.global.debug[info[#info]]
end

function module:ToggleDebug(info, value)
	AddonParent.db.global.debug[info[#info]] = value
end

function module:EnableAllDebug(info)
	for k, v in pairs(AddonParent.db.global.debug) do
		AddonParent.db.global.debug[k] = true
	end
end

function module:DisableAllDebug(info)
	for k, v in pairs(AddonParent.db.global.debug) do
		AddonParent.db.global.debug[k] = false
	end
end

--------------------------------------
--Database Functions for debuging, maybe moved to general use later.
------------------------------------

local host_player, data_name
local function validateFunc(v) return v:upper() end

function module:GetDBO_Options()
	if not AddonParent:GetModule("DBO", true) then return nil end

	local t = {
		name = "Database Functions", order = 2, type = "group", handler = module,
		args = {
			["3pull"] = { 
				name = "3rd Party Data Pull", type = "group", order = 2, inline = true,
				args = {
					Player = { name = "Hosting Player", type = "input", order = 2, get = "HostPlayerGet",
						set = "HostPlayerSet", --[[validate = "NameOnlineValidate"]] },
					data = { name = "Data Player", type = "input", order = 3, get = "DataGet", set = "DataSet", validate = "NameOnlineValidate" },
					getData = { name = "Get Data", type = "execute", order = 4, func = "ExeGet3rdData" },
					info2 = {
						type = "description", order = 5,
						name = ([[ This will allow you to pull someone's data from somebody else. Generally used when that person is offline or is an Alt. ]]):gsub("\009", ""),
					},

				},
			},
			Purge = {
				name = "Purge Player", type = "input", order = 1, get = function() end,
				desc = "Purge Player from database", confirm = true,
				validate = "ValidatePurge",
				set = "SetPurge",
			},
		},
	}
	return t
end

function module:HostPlayerGet(info) return host_player end
function module:HostPlayerSet(info, value) host_player = value:lower():gsub("^%w", validateFunc) end

function module:NameOnlineValidate(info, value)
	value = value:lower():gsub("^%w", validateFunc)
	if AddonParent.online[value] or AddonParent.offline[value] then
		return true
	end
end

function module:DataGet(info) return data_name end
function module:DataSet(info, value) data_name = value:lower():gsub("^%w", validateFunc) end

module["ExeGet3rdData"] = function(self, info)
	if host_player and data_name then
		--if AddonParent.online[host_player] and ( AddonParent.online[data_name] or AddonParent.offline[data_name] ) then
			AddonParent:GetModule("DBO"):SendWhisper(host_player, "DBO_Func_GetData", data_name)
		--end
	else
		AddonParent:GetModule("DBO"):Print("No Hosting Player or Data Player")
	end
end


function module:ValidatePurge(info, value)
	value = value:lower():gsub("^%w", validateFunc)
	return AddonParent.db.factionrealm.links[value] and true or false
end

function module:SetPurge(info, value)
	value = value:lower():gsub("^%w", validateFunc)
	AddonParent.db.factionrealm.links[value] = delTable(AddonParent.db.factionrealm.links[value])
	self:Print(L["%s removed from database"], value)	
end
